import {
  Text, Button, Checkbox, Progress,
} from '@zeedhi/common';
import {
  Loader, Metadata, Http,
  IDictionary, IEventParam,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pla00106AcompanhamentoCusto {
    private utilities = Loader.getInstance('Utilities');
    
    private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
    
    public showWindow = false;
    
    public autoUpdate = false;

    public showAttCusto = false;
    
    public dateSelectedInCard = '';
    
    private planejadoDia: IDictionary = {};
    
    private custoPadrao: IDictionary = {};
    
    private custoPrato: IDictionary = {};
    
    private custoRealizado: IDictionary = {};
    
    public changeVisibilityWindow() {
      this.showWindow = !this.showWindow;
    }
    
    public windowInitial({ event }: IEventParam<Button>) {
      (event as IDictionary).stopPropagation();
      if (this.elaboracaoCardapio.getFilterData.NRCARDPME !== undefined) {
        if (!this.showWindow) {
          this.changeVisibilityWindow();
          this.procedureConsPlan();
        } else {
          this.changeVisibilityWindow();
        }
      } else {
        this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
      }
    }
    
    public autoUpdateAcomp(initialDate?: string, finalDate?: string) {
      if (this.showWindow && this.autoUpdate) this.procedureConsPlan(initialDate, finalDate);
      else if (this.showWindow) this.showAttCusto = true;
      else this.showAttCusto = false;
    }
    
    public procedureConsPlan(initialDate?: string, finalDate?: string) {
      const row = { ...this.elaboracaoCardapio.getFilterData };
      if (initialDate) {
        row.initialDate = initialDate;
        row.finalDate = (finalDate !== undefined) ? finalDate : initialDate;
      }
      this.changeProgress();
      Http.post('/procedureConsPlanPME', { requestType: 'Row', row }).then((response) => {
        if (response.data.dataset.consplan.length > 0) {
          this.showAttCusto = false;
          if (initialDate === undefined) document.getElementsByClassName('progAcomp')[0].scrollIntoView();
          this.makeRequestData(response.data.dataset.consplan[0]);
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
        this.changeProgress();
      });
    }
    
    private makeRequestData(consplan: string) {
      this.requestAllData(consplan).then(() => {
        this.delConsPlan(consplan).then(() => {
          this.requestRealizado().then(() => {
            this.requestCustoPrato().then(() => {
              const cardapio: Cardapio = Metadata.getInstance('cardapio');
              this.custoPadrao = JSON.parse(JSON.stringify(cardapio.dates));
              this.getAllDataEvenFilter();
              this.changeProgress();
            });
          });
        });
      });
    }
    
    private requestAllData(consplan: string) {
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? null : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      const referenceDate = (this.elaboracaoCardapio.getFilterData.referenceDate.length > 0) ? this.elaboracaoCardapio.getFilterData.referenceDate : null;
      this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.elaboracaoCardapio.getFilterData.planned);
      this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.elaboracaoCardapio.getFilterData.NRCARDPME);
      this.utilities.onMountedFilterFind(filter, 'P_DTREFECUSTO', '=', referenceDate);
      this.utilities.onMountedFilterFind(filter, 'P_NRCONSPLAN', '=', consplan);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI);
      this.utilities.onMountedFilterFind(filter, 'P_DTINICIAL', '=', this.elaboracaoCardapio.getFilterData.initialDate);
      this.utilities.onMountedFilterFind(filter, 'P_DTFINAL', '=', this.elaboracaoCardapio.getFilterData.finalDate);
      return Http.post('/find/planejadoDia', { requestType: 'FilterData', filter, limit: 10000 }).then((planejadoDia) => {
        this.planejadoDia = planejadoDia.data.data;
      });
    }
    
    private delConsPlan(consplan: string) {
      const row = { ...this.elaboracaoCardapio.getFilterData };
      row.CONSPLAN = consplan;
      return Http.post('/deleteConsPlan', { requestType: 'Row', row }).then((response) => {
        const hasExclude = response.data.dataset.hasExclude[0];
        return hasExclude;
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
    
    private requestRealizado() {
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? this.elaboracaoCardapio.getFilterData.cdfilcentpme : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      const referenceDate = (this.elaboracaoCardapio.getFilterData.referenceDate.length > 0) ? this.elaboracaoCardapio.getFilterData.referenceDate : null;
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_DTINI', '=', this.elaboracaoCardapio.getFilterData.initialDate);
      this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.elaboracaoCardapio.getFilterData.planned);
      this.utilities.onMountedFilterFind(filter, 'P_DTREFECUSTO', '=', referenceDate);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI);
      this.utilities.onMountedFilterFind(filter, 'P_DTFIN', '=', this.elaboracaoCardapio.getFilterData.finalDate);
      return Http.post('/find/custoRealizado', { requestType: 'FilterData', filter, limit: 10000 }).then((custoRealizado) => {
        this.custoRealizado = custoRealizado.data.data;
      });
    }
    
    private requestCustoPrato() {
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ? this.elaboracaoCardapio.getFilterData.cdfilcentpme : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      const referenceDate = (this.elaboracaoCardapio.getFilterData.referenceDate.length > 0) ? this.elaboracaoCardapio.getFilterData.referenceDate : null;
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.elaboracaoCardapio.getFilterData.planned);
      this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.elaboracaoCardapio.getFilterData.NRCARDPME);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.elaboracaoCardapio.getFilterData.cdfilcentpme);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.elaboracaoCardapio.getFilterData.service.CDSERVFILI);
      this.utilities.onMountedFilterFind(filter, 'P_DTINICIAL', '=', this.elaboracaoCardapio.getFilterData.initialDate);
      this.utilities.onMountedFilterFind(filter, 'P_CDTIPOCARD', '=', this.elaboracaoCardapio.getFilterData.typeMenu.CDTIPOCARD);
      this.utilities.onMountedFilterFind(filter, 'P_DTFINAL', '=', this.elaboracaoCardapio.getFilterData.finalDate);
      this.utilities.onMountedFilterFind(filter, 'P_DTREFERENCIA', '=', referenceDate);
      return Http.post('/find/custoPrato', { requestType: 'FilterData', filter, limit: 1000000 }).then((custoPrato) => {
        this.custoPrato = custoPrato.data.data;
      });
    }
    
    public getAllDataEvenFilter() {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      this.dateSelectedInCard = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
      const filterPlanDia = this.planejadoDia.find((dia: IDictionary) => dia.DTDATA.substring(0, 10) === this.dateSelectedInCard);
      const totalMonths = this.totalMonths();
      const acumulado = this.totalMonths(this.dateSelectedInCard);
      this.setAllTextData(filterPlanDia, totalMonths, acumulado);
    }
    
    private totalMonths(hasDate?: string): IDictionary {
      const copyPlanDia = JSON.parse(JSON.stringify(this.planejadoDia));
      const totalMonths: IDictionary = {};
      totalMonths.VRCUSTPRODMAT = 0; totalMonths.VRCUSTOCONS = 0; totalMonths.CUSTO = 0; totalMonths.PADRAO = 0; totalMonths.COMENSAIS = 0;
      
      if (hasDate === undefined) {
        this.allDays(totalMonths, copyPlanDia);
        totalMonths.CUSTO = totalMonths.VRCUSTPRODMAT + totalMonths.VRCUSTOCONS;
        totalMonths.SALDO = totalMonths.PADRAO - totalMonths.CUSTO;
      } else {
        this.upSelectedData(totalMonths, copyPlanDia);
      }
      
      return totalMonths;
    }
    
    // Soma os dados ate a data selecionada
    private upSelectedData(totalMonths: IDictionary, copyPlanDia: IDictionary) {
      Object.keys(copyPlanDia).forEach((dia: string) => {
        if (copyPlanDia[dia].DTDATA.substring(0, 10) <= this.dateSelectedInCard) {
          totalMonths.VRCUSTPRODMAT += copyPlanDia[dia].VRCUSTPRODMAT;
          totalMonths.VRCUSTOCONS += copyPlanDia[dia].VRCUSTOCONS;
          totalMonths.COMENSAIS += (this.elaboracaoCardapio.getFilterData.paramfilial.IDCOMCUSPERC === 'C') ? copyPlanDia[dia].QTCLIESTICAR : copyPlanDia[dia].QTCOMESTICAR;
        }
      });
    }
    
    // Soma os dados de todos os dias.
    private allDays(totalMonths: IDictionary, copyPlanDia: IDictionary) {
      Object.keys(copyPlanDia).forEach((dia: string) => {
        totalMonths.VRCUSTPRODMAT += copyPlanDia[dia].VRCUSTPRODMAT;
        totalMonths.VRCUSTOCONS += copyPlanDia[dia].VRCUSTOCONS;
        totalMonths.COMENSAIS += (this.elaboracaoCardapio.getFilterData.paramfilial.IDCOMCUSPERC === 'C') ? copyPlanDia[dia].QTCLIESTICAR : copyPlanDia[dia].QTCOMESTICAR;
        totalMonths.PADRAO += (this.custoPadrao.length > 0 && dia < this.custoPadrao.length) ? this.custoPadrao[dia].VRCUSPADTCAR : 0;
      });
    }
    
    private setAllTextData(planDia: IDictionary, totalMonths: IDictionary, acumulado: IDictionary) {
      this.setTextPlanDia(planDia);
      this.setTextPerCapita(planDia);
      this.setTotalMes(totalMonths);
      this.setTextAcumulado(acumulado);
    }
    
    // PLANEJAMENTO DO DIA
    private setTextPlanDia(planDia: IDictionary) {
      Metadata.getInstance<Text>('valueMateriaPlan').text = this.utilities.formatValues(planDia.VRCUSTPRODMAT, 5);
      Metadata.getInstance<Text>('valueConsumoPlan').text = this.utilities.formatValues(planDia.VRCUSTOCONS, 5);
      Metadata.getInstance<Text>('valueCustoTotalPlan').text = this.utilities.formatValues((planDia.VRCUSTPRODMAT + planDia.VRCUSTOCONS), 5);
      
      // REALIZADO
      if (this.custoRealizado.length <= 0) {
        Metadata.getInstance<Text>('valueCustoTotalPlan2').text = this.utilities.formatValues(0, 5);
      }
    }
    
    // PER CAPITA DO DIA
    private setTextPerCapita(planDia: IDictionary) {
      const newMat = (this.elaboracaoCardapio.getFilterData.paramfilial.IDCOMCUSPERC === 'C') ? this.calcValue(planDia.VRCUSTPRODMAT, planDia.QTCLIESTICAR) : this.calcValue(planDia.VRCUSTPRODMAT, planDia.QTCOMESTICAR);
      const newCons = (this.elaboracaoCardapio.getFilterData.paramfilial.IDCOMCUSPERC === 'C') ? this.calcValue(planDia.VRCUSTOCONS, planDia.QTCLIESTICAR) : this.calcValue(planDia.VRCUSTOCONS, planDia.QTCOMESTICAR);
      const custo = (newMat + newCons);
      const saldo = (this.getCustPadraoSelected() - custo);
      Metadata.getInstance<Text>('valueMateria').text = this.utilities.formatValues(newMat, 5);
      Metadata.getInstance<Text>('valueConsumo').text = this.utilities.formatValues(newCons, 5);
      Metadata.getInstance<Text>('valueCusto').text = this.utilities.formatValues(custo, 5);
      Metadata.getInstance<Text>('valuePadrao').text = this.utilities.formatValues(this.getCustPadraoSelected(), 5);
      Metadata.getInstance<Text>('valueSaldo').text = this.utilities.formatValues(saldo, 5);
      if (saldo < 0) document.getElementsByName('valueSaldo')[0].style.color = 'red';
      else document.getElementsByName('valueSaldo')[0].style.color = 'var(--zd-font-color)';
    }
    
    // PER CAPITA DO DIA TOTAL MES
    private setTotalMes(totalMonths: IDictionary) {
      Metadata.getInstance<Text>('valueMateria2').text = this.utilities.formatValues(totalMonths.VRCUSTPRODMAT, 5);
      Metadata.getInstance<Text>('valueConsumo2').text = this.utilities.formatValues(totalMonths.VRCUSTOCONS, 5);
      Metadata.getInstance<Text>('valueCusto2').text = this.utilities.formatValues(totalMonths.CUSTO, 5);
      Metadata.getInstance<Text>('valuePadrao2').text = this.utilities.formatValues(totalMonths.PADRAO, 5);
      Metadata.getInstance<Text>('valueSaldo2').text = this.utilities.formatValues(totalMonths.SALDO, 5);
      if (totalMonths.SALDO < 0) document.getElementsByName('valueSaldo2')[0].style.color = 'red';
      else document.getElementsByName('valueSaldo2')[0].style.color = 'var(--zd-font-color)';
    }
    
    // ACUMULADO ATÉ
    private setTextAcumulado(acumulado: IDictionary) {
      const custoTotal = acumulado.VRCUSTPRODMAT + acumulado.VRCUSTOCONS;
      Metadata.getInstance<Text>('valueMateriaAcu').text = this.utilities.formatValues(acumulado.VRCUSTPRODMAT, 5);
      Metadata.getInstance<Text>('valueConsumoAcu').text = this.utilities.formatValues(acumulado.VRCUSTOCONS, 5);
      Metadata.getInstance<Text>('valueCustoTotalAcu').text = this.utilities.formatValues(custoTotal, 5);
      Metadata.getInstance<Text>('valueComensaisAcu').text = this.utilities.formatValues(acumulado.COMENSAIS, 5);
      Metadata.getInstance<Text>('valuePCMAcu').text = this.utilities.formatValues(this.calcValue(custoTotal, acumulado.COMENSAIS), 5);
      
      // REALIZADO
      if (this.custoRealizado.length <= 0) {
        Metadata.getInstance<Text>('valueMateriaAcu2').text = this.utilities.formatValues(0, 5);
        Metadata.getInstance<Text>('valueConsumoAcu2').text = this.utilities.formatValues(0, 5);
        Metadata.getInstance<Text>('valueCustoTotalAcu2').text = this.utilities.formatValues(0, 5); // Soma de CUSTO até a data posicionada
        Metadata.getInstance<Text>('valueComensaisAcu2').text = this.utilities.formatValues(0, 5); // Soma de QTCOMEFETCAR até a data posicionada
        Metadata.getInstance<Text>('valuePCMAcu2').text = this.utilities.formatValues(0, 5); // Custo total / comensais somado
      }
    }
    
    private calcValue(vrCust: number, qt: number): number {
      if (vrCust > 0 && qt > 0) return (vrCust / qt);
      return vrCust;
    }
    
    private calcValuePCM(vrCust: number, qt: number): number {
      if (vrCust > 0 && qt > 0) return (vrCust / qt);
      return vrCust;
    }
    
    private getCustPadraoSelected(): number {
      if (this.custoPadrao.length > 0) {
        const custo = this.custoPadrao.find((dia: IDictionary) => dia.DTCARDIAPME.substring(0, 10) === this.dateSelectedInCard);
        if (custo) return custo.VRCUSPADTCAR;
      }
      return 0;
    }
    
    public controllReloadDisable({ component }: IEventParam<Checkbox>) {
      const reload = Metadata.getInstance<Button>('reloadCusto');
      reload.disabled = !reload.disabled;
      this.autoUpdate = component.value;
    }
    
    public reload() {
      this.procedureConsPlan();
    }
    
    public changeProgress() {
      const progress = Metadata.getInstance<Progress>('progressAcomp');
      progress.isVisible = !progress.isVisible;
    }
}
