import {
  Metadata, Loader, IEventParam, Http, IDictionary, Datasource,
} from '@zeedhi/core';

import {
  Select, Grid, Tabs, Form, Card,
} from '@zeedhi/common';

export class Pme00500ComensaisEstimadosController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public newRow = false;
  
  public editable = false;
  
  private armazenaForm: IDictionary = {};
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
    card.isVisible = false;
  }
  
  public setGridComensaisUnAgreg() {
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const grid: Grid = Metadata.getInstance('gridComensaisUnAgreg');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value ? unidade.value : '',
      },
      {
        name: 'NRORG',
        operator: '=',
        value: localStorage.getItem('NRORG'),
      },
    ];
    
    grid.datasource.filter = filter;
    grid.datasource.get();
  }
  
  public onChangeGridComensaisCurrentRow({ component }: IEventParam<Datasource>) {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const row = component.currentRow;
    Http.post('/comensaisUnAgreg/dadosComensais', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset?.result[0];
      if (result && response.data.dataset.result.length !== 0) {
        Object.keys(result).forEach((item) => {
          if (item in form.value) {
            form.value[item] = result[item];
          }
        });
        form.value.__is_new = false;
      } else {
        Object.keys(form.value).forEach((key) => {
          if (key.includes('QT')) {
            form.value[key] = 0;
          }
        });
        form.value.__is_new = true;
      }
      this.armazenaForm = JSON.parse(JSON.stringify(form.value));
    });
  }
  
  public cardIsVisible() {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    
    card.isVisible = true;
  }

  public saveForm() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const grid: Grid = Metadata.getInstance('gridComensaisUnAgreg');
    const rowGrid = grid.datasource.currentRow;
    const row = form.value;
    if (form.validate()) {
      if (row.EDITED) {
        if (row.CDFILCENTPME === '') {
          row.CDFILCENTPME = rowGrid.CDFILCENTPME;
          row.CDFILIAL = rowGrid.CDFILIAL;
          row.CDSERVFILI = rowGrid.CDSERVFILI;
          row.CDTIPOCARD = rowGrid.CDTIPOCARD;
        }
        
        Http.post('/comensaisUnAgreg/save', { requestType: 'Row', row }).then((response) => {
          if (response.data.dataset.save[0]) {
            this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
            grid.datasource.get().then(() => {
              grid.datasource.currentRow = rowGrid;
            });
            row.EDITED = false;
            form.value.__is_new = false;
            this.armazenaForm = JSON.parse(JSON.stringify(form.value));
          }
        });
      } else this.utilities.showAlert('Alert', 'pme::nenhumRegEditado', 'warning');
    }
  }

  public cancelForm() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');

    const valueForm = Object.entries(form.value);
    const valueConst = Object.entries(this.armazenaForm);
 
    const validate = (valueForm.length === valueConst.length) && valueForm.every((element, index) => element === valueConst[index]);
    
    if (validate === false) {
      Object.keys(this.armazenaForm).forEach((item: string) => {
        form.value[item] = this.armazenaForm[item];
      });
    }
  }
  
  public setValuesTotal() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const valueForm = form.value;

    valueForm.EDITED = true;
    valueForm.QTCOMPMESEG = valueForm.QTCLIPMESEG + valueForm.QTFUNPMESEG + valueForm.QTDOAPMESEG;
    valueForm.QTCOMPMETER = valueForm.QTCLIPMETER + valueForm.QTFUNPMETER + valueForm.QTDOAPMETER;
    valueForm.QTCOMPMEQUA = valueForm.QTCLIPMEQUA + valueForm.QTFUNPMEQUA + valueForm.QTDOAPMEQUA;
    valueForm.QTCOMPMEQUI = valueForm.QTCLIPMEQUI + valueForm.QTFUNPMEQUI + valueForm.QTDOAPMEQUI;
    valueForm.QTCOMPMESEX = valueForm.QTCLIPMESEX + valueForm.QTFUNPMESEX + valueForm.QTDOAPMESEX;
    valueForm.QTCOMPMESAB = valueForm.QTCLIPMESAB + valueForm.QTFUNPMESAB + valueForm.QTDOAPMESAB;
    valueForm.QTCOMPMEDOM = valueForm.QTCLIPMEDOM + valueForm.QTFUNPMEDOM + valueForm.QTDOAPMEDOM;
    valueForm.QTCOMPMEFER = valueForm.QTCLIPMEFER + valueForm.QTFUNPMEFER + valueForm.QTDOAPMEFER;
  }

  // Funções segunda tab.
  public filterGrids({ component }: IEventParam<Tabs>) {
    if (component.activeTab === 1) this.filterGridConsolidado();
  }
  
  public filterGridConsolidado() {
    const grid: Grid = Metadata.getInstance('gridConsolidadoCentral');
    
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'P_NRORG',
        operator: '=',
        value: localStorage.getItem('NRORG'),
      },
    ];
    
    grid.datasource.filter = filter;
    grid.datasource.get();
  }
  
  public onChangeGridConsolidadoCurrentRow({ component }: IEventParam<Grid>) {
    const form: Form = Metadata.getInstance('formConsolidadoCentral');
    const card: Card = Metadata.getInstance('formConsolidadoCentral-card');
    const row = component.datasource.currentRow;
    Http.post('/consolidadoCentral/dadosConsolidado', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result[0];
      if (response.data.dataset.result.length !== 0) {
        Object.keys(result).forEach((item) => {
          if (item in form.value) {
            form.value[item] = result[item];
          }
        });
      }
      this.armazenaForm = JSON.parse(JSON.stringify(form.value));
    });
    card.isVisible = true;
  }

  public closeCardComensais() {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    card.isVisible = false;
  }
}
